@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 7                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2018 The PHP Group                                |
+   | Copyright (c) The PHP Group                                          |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -17,8 +17,6 @@
    +----------------------------------------------------------------------+
  */
 
-/* $Id$ */
-
 /* {{{ includes */
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -60,11 +58,6 @@ MBSTRING_API SAPI_TREAT_DATA_FUNC(mbstr_treat_data)
 	const mbfl_encoding *detected;
 	php_mb_encoding_handler_info_t info;
 
-	if (arg != PARSE_STRING) {
-		char *value = MBSTRG(internal_encoding_name);
-		_php_mb_ini_mbstring_internal_encoding_set(value, value ? strlen(value): 0);
-	}
-
 	if (!MBSTRG(encoding_translation)) {
 		php_default_treat_data(arg, str, destArray);
 		return;
@@ -92,30 +85,28 @@ MBSTRING_API SAPI_TREAT_DATA_FUNC(mbstr_treat_data)
 			break;
 	}
 
-	if (arg == PARSE_POST) {
-		sapi_handle_post(&v_array);
-		return;
-	}
-
-	if (arg == PARSE_GET) {		/* GET data */
-		c_var = SG(request_info).query_string;
-		if (c_var && *c_var) {
-			res = (char *) estrdup(c_var);
+	switch (arg) {
+		case PARSE_POST:
+			sapi_handle_post(&v_array);
+			return;
+		case PARSE_GET: /* GET data */
+			c_var = SG(request_info).query_string;
+			if (c_var && *c_var) {
+				res = (char *) estrdup(c_var);
+				free_buffer = 1;
+			}
+			break;
+		case PARSE_COOKIE: /* Cookie data */
+			c_var = SG(request_info).cookie_data;
+			if (c_var && *c_var) {
+				res = (char *) estrdup(c_var);
+				free_buffer = 1;
+			}
+			break;
+		case PARSE_STRING: /* String data */
+			res = str;
 			free_buffer = 1;
-		} else {
-			free_buffer = 0;
-		}
-	} else if (arg == PARSE_COOKIE) {		/* Cookie data */
-		c_var = SG(request_info).cookie_data;
-		if (c_var && *c_var) {
-			res = (char *) estrdup(c_var);
-			free_buffer = 1;
-		} else {
-			free_buffer = 0;
-		}
-	} else if (arg == PARSE_STRING) {		/* String data */
-		res = str;
-		free_buffer = 1;
+			break;
 	}
 
 	if (!res) {
@@ -196,16 +187,16 @@ const mbfl_encoding *_php_mb_encoding_handler_ex(const php_mb_encoding_handler_i
 	const char *s1, *s2;
 	char *strtok_buf = NULL, **val_list = NULL;
 	zval *array_ptr = (zval *) arg;
-	int n, num, *len_list = NULL;
+	size_t n, num, *len_list = NULL;
 	size_t val_len, new_val_len;
 	mbfl_string string, resvar, resval;
 	const mbfl_encoding *from_encoding = NULL;
 	mbfl_encoding_detector *identd = NULL;
 	mbfl_buffer_converter *convd = NULL;
 
-	mbfl_string_init_set(&string, info->to_language, info->to_encoding->no_encoding);
-	mbfl_string_init_set(&resvar, info->to_language, info->to_encoding->no_encoding);
-	mbfl_string_init_set(&resval, info->to_language, info->to_encoding->no_encoding);
+	mbfl_string_init_set(&string, info->to_language, info->to_encoding);
+	mbfl_string_init_set(&resvar, info->to_language, info->to_encoding);
+	mbfl_string_init_set(&resval, info->to_language, info->to_encoding);
 
 	if (!res || *res == '\0') {
 		goto out;
@@ -225,7 +216,7 @@ const mbfl_encoding *_php_mb_encoding_handler_ex(const php_mb_encoding_handler_i
 	num *= 2; /* need space for variable name and value */
 
 	val_list = (char **)ecalloc(num, sizeof(char *));
-	len_list = (int *)ecalloc(num, sizeof(int));
+	len_list = (size_t *)ecalloc(num, sizeof(size_t));
 
 	/* split and decode the query */
 	n = 0;
@@ -253,7 +244,7 @@ const mbfl_encoding *_php_mb_encoding_handler_ex(const php_mb_encoding_handler_i
 		var = php_strtok_r(NULL, info->separator, &strtok_buf);
 	}
 
-	if (n > (PG(max_input_vars) * 2)) {
+	if (ZEND_SIZE_T_GT_ZEND_LONG(n, (PG(max_input_vars) * 2))) {
 		php_error_docref(NULL, E_WARNING, "Input variables exceeded " ZEND_LONG_FMT ". To increase the limit change max_input_vars in php.ini.", PG(max_input_vars));
 		goto out;
 	}
@@ -261,14 +252,14 @@ const mbfl_encoding *_php_mb_encoding_handler_ex(const php_mb_encoding_handler_i
 	num = n; /* make sure to process initialized vars only */
 
 	/* initialize converter */
-	if (info->num_from_encodings <= 0) {
+	if (info->num_from_encodings == 0) {
 		from_encoding = &mbfl_encoding_pass;
 	} else if (info->num_from_encodings == 1) {
 		from_encoding = info->from_encodings[0];
 	} else {
 		/* auto detect */
 		from_encoding = NULL;
-		identd = mbfl_encoding_detector_new2(info->from_encodings, info->num_from_encodings, MBSTRG(strict_detection));
+		identd = mbfl_encoding_detector_new(info->from_encodings, info->num_from_encodings, MBSTRG(strict_detection));
 		if (identd != NULL) {
 			n = 0;
 			while (n < num) {
@@ -279,7 +270,7 @@ const mbfl_encoding *_php_mb_encoding_handler_ex(const php_mb_encoding_handler_i
 				}
 				n++;
 			}
-			from_encoding = mbfl_encoding_detector_judge2(identd);
+			from_encoding = mbfl_encoding_detector_judge(identd);
 			mbfl_encoding_detector_delete(identd);
 		}
 		if (!from_encoding) {
@@ -292,7 +283,7 @@ const mbfl_encoding *_php_mb_encoding_handler_ex(const php_mb_encoding_handler_i
 
 	convd = NULL;
 	if (from_encoding != &mbfl_encoding_pass) {
-		convd = mbfl_buffer_converter_new2(from_encoding, info->to_encoding, 0);
+		convd = mbfl_buffer_converter_new(from_encoding, info->to_encoding, 0);
 		if (convd != NULL) {
 			mbfl_buffer_converter_illegal_mode(convd, MBSTRG(current_filter_illegal_mode));
 			mbfl_buffer_converter_illegal_substchar(convd, MBSTRG(current_filter_illegal_substchar));
@@ -305,7 +296,7 @@ const mbfl_encoding *_php_mb_encoding_handler_ex(const php_mb_encoding_handler_i
 	}
 
 	/* convert encoding */
-	string.no_encoding = from_encoding->no_encoding;
+	string.encoding = from_encoding;
 
 	n = 0;
 	while (n < num) {
@@ -379,7 +370,7 @@ SAPI_POST_HANDLER_FUNC(php_mb_post_handler)
 	post_data_str = php_stream_copy_to_mem(SG(request_info).request_body, PHP_STREAM_COPY_ALL, 0);
 	detected = _php_mb_encoding_handler_ex(&info, arg, post_data_str ? ZSTR_VAL(post_data_str) : NULL);
 	if (post_data_str) {
-		zend_string_release(post_data_str);
+		zend_string_release_ex(post_data_str, 0);
 	}
 
 	MBSTRG(http_input_identify) = detected;
@@ -390,13 +381,3 @@ SAPI_POST_HANDLER_FUNC(php_mb_post_handler)
 /* }}} */
 
 #endif /* HAVE_MBSTRING */
-
-/*
- * Local variables:
- * tab-width: 4
- * c-basic-offset: 4
- * End:
- * vim600: fdm=marker
- * vim: noet sw=4 ts=4
- */
-
