@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | Zend OPcache                                                         |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1998-2018 The PHP Group                                |
+   | Copyright (c) The PHP Group                                          |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -12,10 +12,10 @@
    | obtain it through the world-wide-web, please send a note to          |
    | license@php.net so we can mail you a copy immediately.               |
    +----------------------------------------------------------------------+
-   | Authors: Andi Gutmans <andi@zend.com>                                |
-   |          Zeev Suraski <zeev@zend.com>                                |
+   | Authors: Andi Gutmans <andi@php.net>                                 |
+   |          Zeev Suraski <zeev@php.net>                                 |
    |          Stanislav Malyshev <stas@zend.com>                          |
-   |          Dmitry Stogov <dmitry@zend.com>                             |
+   |          Dmitry Stogov <dmitry@php.net>                              |
    +----------------------------------------------------------------------+
 */
 
@@ -28,7 +28,6 @@
 #include <fcntl.h>
 #ifndef ZEND_WIN32
 # include <sys/types.h>
-# include <dirent.h>
 # include <signal.h>
 # include <sys/stat.h>
 # include <stdio.h>
@@ -260,6 +259,7 @@ int zend_shared_alloc_startup(size_t requested_size)
 void zend_shared_alloc_shutdown(void)
 {
 	zend_shared_segment **tmp_shared_segments;
+	zend_shared_segment *shared_segments_buf[16];
 	size_t shared_segments_array_size;
 	zend_smm_shared_globals tmp_shared_globals;
 	int i;
@@ -267,18 +267,28 @@ void zend_shared_alloc_shutdown(void)
 	tmp_shared_globals = *smm_shared_globals;
 	smm_shared_globals = &tmp_shared_globals;
 	shared_segments_array_size = ZSMMG(shared_segments_count) * (S_H(segment_type_size)() + sizeof(void *));
-	tmp_shared_segments = emalloc(shared_segments_array_size);
+	if (shared_segments_array_size > 16) {
+		tmp_shared_segments = malloc(shared_segments_array_size);
+	} else {
+		tmp_shared_segments = shared_segments_buf;
+	}
 	copy_shared_segments(tmp_shared_segments, ZSMMG(shared_segments)[0], ZSMMG(shared_segments_count), S_H(segment_type_size)());
 	ZSMMG(shared_segments) = tmp_shared_segments;
 
 	for (i = 0; i < ZSMMG(shared_segments_count); i++) {
 		S_H(detach_segment)(ZSMMG(shared_segments)[i]);
 	}
-	efree(ZSMMG(shared_segments));
+	if (shared_segments_array_size > 16) {
+		free(ZSMMG(shared_segments));
+	}
 	ZSMMG(shared_segments) = NULL;
 	g_shared_alloc_handler = NULL;
 #ifndef ZEND_WIN32
 	close(lock_file);
+
+# ifdef ZTS
+	tsrm_mutex_free(zts_lock);
+# endif
 #endif
 }
 
@@ -337,33 +347,91 @@ void *zend_shared_alloc(size_t size)
 int zend_shared_memdup_size(void *source, size_t size)
 {
 	void *old_p;
+	zend_ulong key = (zend_ulong)source;
 
-	if ((old_p = zend_hash_index_find_ptr(&ZCG(xlat_table), (zend_ulong)source)) != NULL) {
+	key = (key >> 3) | (key << ((sizeof(key) * 8) - 3)); /* key  = _rotr(key, 3);*/
+	if ((old_p = zend_hash_index_find_ptr(&ZCG(xlat_table), key)) != NULL) {
 		/* we already duplicated this pointer */
 		return 0;
 	}
-	zend_shared_alloc_register_xlat_entry(source, source);
+	zend_hash_index_add_new_ptr(&ZCG(xlat_table), key, source);
 	return ZEND_ALIGNED_SIZE(size);
 }
 
-void *_zend_shared_memdup(void *source, size_t size, zend_bool free_source)
+static zend_always_inline void *_zend_shared_memdup(void *source, size_t size, zend_bool arena, zend_bool get_xlat, zend_bool set_xlat, zend_bool free_source)
 {
 	void *old_p, *retval;
+	zend_ulong key;
 
-	if ((old_p = zend_hash_index_find_ptr(&ZCG(xlat_table), (zend_ulong)source)) != NULL) {
-		/* we already duplicated this pointer */
-		return old_p;
+	if (get_xlat) {
+		key = (zend_ulong)source;
+		key = (key >> 3) | (key << ((sizeof(key) * 8) - 3)); /* key  = _rotr(key, 3);*/
+		if ((old_p = zend_hash_index_find_ptr(&ZCG(xlat_table), key)) != NULL) {
+			/* we already duplicated this pointer */
+			return old_p;
+		}
+	}
+	if (arena) {
+		retval = ZCG(arena_mem);
+		ZCG(arena_mem) = (void*)(((char*)ZCG(arena_mem)) + ZEND_ALIGNED_SIZE(size));
+	} else {
+		retval = ZCG(mem);
+		ZCG(mem) = (void*)(((char*)ZCG(mem)) + ZEND_ALIGNED_SIZE(size));
 	}
-	retval = ZCG(mem);
-	ZCG(mem) = (void*)(((char*)ZCG(mem)) + ZEND_ALIGNED_SIZE(size));
 	memcpy(retval, source, size);
-	zend_shared_alloc_register_xlat_entry(source, retval);
+	if (set_xlat) {
+		if (!get_xlat) {
+			key = (zend_ulong)source;
+			key = (key >> 3) | (key << ((sizeof(key) * 8) - 3)); /* key  = _rotr(key, 3);*/
+		}
+		zend_hash_index_add_new_ptr(&ZCG(xlat_table), key, retval);
+	}
 	if (free_source) {
 		efree(source);
 	}
 	return retval;
 }
 
+void *zend_shared_memdup_get_put_free(void *source, size_t size)
+{
+	return _zend_shared_memdup(source, size, 0, 1, 1, 1);
+}
+
+void *zend_shared_memdup_put_free(void *source, size_t size)
+{
+	return _zend_shared_memdup(source, size, 0, 0, 1, 1);
+}
+
+void *zend_shared_memdup_free(void *source, size_t size)
+{
+	return _zend_shared_memdup(source, size, 0, 0, 0, 1);
+}
+
+void *zend_shared_memdup_get_put(void *source, size_t size)
+{
+	return _zend_shared_memdup(source, size, 0, 1, 1, 0);
+}
+
+void *zend_shared_memdup_put(void *source, size_t size)
+{
+	return _zend_shared_memdup(source, size, 0, 0, 1, 0);
+}
+
+void *zend_shared_memdup(void *source, size_t size)
+{
+	return _zend_shared_memdup(source, size, 0, 0, 0, 0);
+}
+
+void *zend_shared_memdup_arena_put(void *source, size_t size)
+{
+	return _zend_shared_memdup(source, size, 1, 0, 1, 0);
+}
+
+void *zend_shared_memdup_arena(void *source, size_t size)
+{
+	return _zend_shared_memdup(source, size, 1, 0, 0, 0);
+}
+
 void zend_shared_alloc_safe_unlock(void)
 {
 	if (ZCG(locked)) {
@@ -450,16 +518,31 @@ void zend_shared_alloc_clear_xlat_table(void)
 	zend_hash_clean(&ZCG(xlat_table));
 }
 
+uint32_t zend_shared_alloc_checkpoint_xlat_table(void)
+{
+	return ZCG(xlat_table).nNumUsed;
+}
+
+void zend_shared_alloc_restore_xlat_table(uint32_t checkpoint)
+{
+	zend_hash_discard(&ZCG(xlat_table), checkpoint);
+}
+
 void zend_shared_alloc_register_xlat_entry(const void *old, const void *new)
 {
-	zend_hash_index_add_new_ptr(&ZCG(xlat_table), (zend_ulong)old, (void*)new);
+	zend_ulong key = (zend_ulong)old;
+
+	key = (key >> 3) | (key << ((sizeof(key) * 8) - 3)); /* key  = _rotr(key, 3);*/
+	zend_hash_index_add_new_ptr(&ZCG(xlat_table), key, (void*)new);
 }
 
 void *zend_shared_alloc_get_xlat_entry(const void *old)
 {
 	void *retval;
+	zend_ulong key = (zend_ulong)old;
 
-	if ((retval = zend_hash_index_find_ptr(&ZCG(xlat_table), (zend_ulong)old)) == NULL) {
+	key = (key >> 3) | (key << ((sizeof(key) * 8) - 3)); /* key  = _rotr(key, 3);*/
+	if ((retval = zend_hash_index_find_ptr(&ZCG(xlat_table), key)) == NULL) {
 		return NULL;
 	}
 	return retval;
@@ -515,6 +598,25 @@ void zend_accel_shared_protect(int mode)
 	for (i = 0; i < ZSMMG(shared_segments_count); i++) {
 		mprotect(ZSMMG(shared_segments)[i]->p, ZSMMG(shared_segments)[i]->size, mode);
 	}
+#elif defined(ZEND_WIN32)
+	int i;
+
+	if (!smm_shared_globals) {
+		return;
+	}
+
+	if (mode) {
+		mode = PAGE_READONLY;
+	} else {
+		mode = PAGE_READWRITE;
+	}
+
+	for (i = 0; i < ZSMMG(shared_segments_count); i++) {
+		DWORD oldProtect;
+		if (!VirtualProtect(ZSMMG(shared_segments)[i]->p, ZSMMG(shared_segments)[i]->size, mode, &oldProtect)) {
+			zend_accel_error(ACCEL_LOG_ERROR, "Failed to protect memory");
+		}
+	}
 #endif
 }
 
