@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 7                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2018 The PHP Group                                |
+   | Copyright (c) The PHP Group                                          |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -284,8 +284,7 @@ static int ps_files_write(ps_files *data, zend_string *key, zend_string *val)
 static int ps_files_cleanup_dir(const char *dirname, zend_long maxlifetime)
 {
 	DIR *dir;
-	char dentry[sizeof(struct dirent) + MAXPATHLEN];
-	struct dirent *entry = (struct dirent *) &dentry;
+	struct dirent *entry;
 	zend_stat_t sbuf;
 	char buf[MAXPATHLEN];
 	time_t now;
@@ -312,7 +311,7 @@ static int ps_files_cleanup_dir(const char *dirname, zend_long maxlifetime)
 	memcpy(buf, dirname, dirname_len);
 	buf[dirname_len] = PHP_DIR_SEPARATOR;
 
-	while (php_readdir_r(dir, (struct dirent *) dentry, &entry) == 0 && entry) {
+	while ((entry = readdir(dir))) {
 		/* does the file start with our prefix? */
 		if (!strncmp(entry->d_name, FILE_PREFIX, sizeof(FILE_PREFIX) - 1)) {
 			size_t entry_len = strlen(entry->d_name);
@@ -571,8 +570,6 @@ PS_WRITE_FUNC(files)
 PS_UPDATE_TIMESTAMP_FUNC(files)
 {
 	char buf[MAXPATHLEN];
-	struct utimbuf newtimebuf;
-	struct utimbuf *newtime = &newtimebuf;
 	int ret;
 	PS_FILES_DATA;
 
@@ -581,12 +578,7 @@ PS_UPDATE_TIMESTAMP_FUNC(files)
 	}
 
 	/* Update mtime */
-#ifdef HAVE_UTIME_NULL
-	newtime = NULL;
-#else
-	newtime->modtime = newtime->actime = time(NULL);
-#endif
-	ret = VCWD_UTIME(buf, newtime);
+	ret = VCWD_UTIME(buf, NULL);
 	if (ret == -1) {
 		/* New session ID, create data file */
 		return ps_files_write(data, key, val);
@@ -720,12 +712,3 @@ PS_VALIDATE_SID_FUNC(files)
 
 	return ps_files_key_exists(data, ZSTR_VAL(key));
 }
-
-/*
- * Local variables:
- * tab-width: 4
- * c-basic-offset: 4
- * End:
- * vim600: sw=4 ts=4 fdm=marker
- * vim<600: sw=4 ts=4
- */
