@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 7                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2018 The PHP Group                                |
+   | Copyright (c) The PHP Group                                          |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -463,8 +463,9 @@ static int _php_mcast_join_leave(
 			sizeof(greq));
 #else
 	if (sock->type == AF_INET) {
-		struct ip_mreq mreq = {0};
+		struct ip_mreq mreq;
 		struct in_addr addr;
+		memset(&mreq, 0, sizeof(struct ip_mreq));
 
 		assert(group_len == sizeof(struct sockaddr_in));
 
@@ -483,7 +484,8 @@ static int _php_mcast_join_leave(
 	}
 #if HAVE_IPV6
 	else if (sock->type == AF_INET6) {
-		struct ipv6_mreq mreq = {0};
+		struct ipv6_mreq mreq;
+		memset(&mreq, 0, sizeof(struct ipv6_mreq));
 
 		assert(group_len == sizeof(struct sockaddr_in6));
 
@@ -792,28 +794,27 @@ int php_add4_to_if_index(struct in_addr *addr, php_socket *php_sock, unsigned *i
 	for (p = if_conf.ifc_buf;
 		 p < if_conf.ifc_buf + if_conf.ifc_len;
 		 p += entry_len) {
-		struct ifreq *cur_req;
-
-		/* let's hope the pointer is aligned */
-		cur_req = (struct ifreq*) p;
+		/* p may be misaligned on macos. */
+		struct ifreq cur_req;
+		memcpy(&cur_req, p, sizeof(struct ifreq));
 
 #ifdef HAVE_SOCKADDR_SA_LEN
-		entry_len = cur_req->ifr_addr.sa_len + sizeof(cur_req->ifr_name);
+		entry_len = cur_req.ifr_addr.sa_len + sizeof(cur_req.ifr_name);
 #else
 		/* if there's no sa_len, assume the ifr_addr field is a sockaddr */
-		entry_len = sizeof(struct sockaddr) + sizeof(cur_req->ifr_name);
+		entry_len = sizeof(struct sockaddr) + sizeof(cur_req.ifr_name);
 #endif
-		entry_len = MAX(entry_len, sizeof(*cur_req));
+		entry_len = MAX(entry_len, sizeof(cur_req));
 
-		if ((((struct sockaddr*)&cur_req->ifr_addr)->sa_family == AF_INET) &&
-				(((struct sockaddr_in*)&cur_req->ifr_addr)->sin_addr.s_addr ==
+		if ((((struct sockaddr*)&cur_req.ifr_addr)->sa_family == AF_INET) &&
+				(((struct sockaddr_in*)&cur_req.ifr_addr)->sin_addr.s_addr ==
 					addr->s_addr)) {
 #if defined(SIOCGIFINDEX)
-			if (ioctl(php_sock->bsd_socket, SIOCGIFINDEX, (char*)cur_req)
+			if (ioctl(php_sock->bsd_socket, SIOCGIFINDEX, (char*)&cur_req)
 					== -1) {
 #elif defined(HAVE_IF_NAMETOINDEX)
 			unsigned index_tmp;
-			if ((index_tmp = if_nametoindex(cur_req->ifr_name)) == 0) {
+			if ((index_tmp = if_nametoindex(cur_req.ifr_name)) == 0) {
 #else
 #error Neither SIOCGIFINDEX nor if_nametoindex are available
 #endif
@@ -823,7 +824,7 @@ int php_add4_to_if_index(struct in_addr *addr, php_socket *php_sock, unsigned *i
 				goto err;
 			} else {
 #if defined(SIOCGIFINDEX)
-				*if_index = cur_req->ifr_ifindex;
+				*if_index = cur_req.ifr_ifindex;
 #else
 				*if_index = index_tmp;
 #endif
