@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 7                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2018 The PHP Group                                |
+   | Copyright (c) The PHP Group                                          |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -15,7 +15,6 @@
    | Author: Rasmus Lerdorf <rasmus@lerdorf.on.ca>                        |
    +----------------------------------------------------------------------+
  */
-/* $Id$ */
 
 #include <stdio.h>
 #include "php.h"
@@ -24,13 +23,10 @@
 #include "SAPI.h"
 #include "php_main.h"
 #include "head.h"
-#ifdef TM_IN_SYS_TIME
-#include <sys/time.h>
-#else
 #include <time.h>
-#endif
 
 #include "php_globals.h"
+#include "zend_smart_str.h"
 
 
 /* Implementation of the language Header() function */
@@ -80,15 +76,12 @@ PHPAPI int php_header(void)
 	}
 }
 
-
-PHPAPI int php_setcookie(zend_string *name, zend_string *value, time_t expires, zend_string *path, zend_string *domain, int secure, int url_encode, int httponly)
+PHPAPI int php_setcookie(zend_string *name, zend_string *value, time_t expires, zend_string *path, zend_string *domain, int secure, int httponly, zend_string *samesite, int url_encode)
 {
-	char *cookie;
-	size_t len = sizeof("Set-Cookie: ");
 	zend_string *dt;
 	sapi_header_line ctr = {0};
 	int result;
-	zend_string *encoded_value = NULL;
+	smart_str buf = {0};
 
 	if (!ZSTR_LEN(name)) {
 		zend_error( E_WARNING, "Cookie names must not be empty" );
@@ -104,25 +97,15 @@ PHPAPI int php_setcookie(zend_string *name, zend_string *value, time_t expires,
 		return FAILURE;
 	}
 
-	len += ZSTR_LEN(name);
-	if (value) {
-		if (url_encode) {
-			encoded_value = php_url_encode(ZSTR_VAL(value), ZSTR_LEN(value));
-			len += ZSTR_LEN(encoded_value);
-		} else {
-			encoded_value = zend_string_copy(value);
-			len += ZSTR_LEN(encoded_value);
-		}
+	if (path && strpbrk(ZSTR_VAL(path), ",; \t\r\n\013\014") != NULL) { /* man isspace for \013 and \014 */
+		zend_error(E_WARNING, "Cookie paths cannot contain any of the following ',; \\t\\r\\n\\013\\014'" );
+		return FAILURE;
 	}
 
-	if (path) {
-		len += ZSTR_LEN(path);
+	if (domain && strpbrk(ZSTR_VAL(domain), ",; \t\r\n\013\014") != NULL) { /* man isspace for \013 and \014 */
+		zend_error(E_WARNING, "Cookie domains cannot contain any of the following ',; \\t\\r\\n\\013\\014'" );
+		return FAILURE;
 	}
-	if (domain) {
-		len += ZSTR_LEN(domain);
-	}
-
-	cookie = emalloc(len + 100);
 
 	if (value == NULL || ZSTR_LEN(value) == 0) {
 		/*
@@ -131,73 +114,124 @@ PHPAPI int php_setcookie(zend_string *name, zend_string *value, time_t expires,
 		 * pick an expiry date in the past
 		 */
 		dt = php_format_date("D, d-M-Y H:i:s T", sizeof("D, d-M-Y H:i:s T")-1, 1, 0);
-		snprintf(cookie, len + 100, "Set-Cookie: %s=deleted; expires=%s; Max-Age=0", ZSTR_VAL(name), ZSTR_VAL(dt));
+		smart_str_appends(&buf, "Set-Cookie: ");
+		smart_str_append(&buf, name);
+		smart_str_appends(&buf, "=deleted; expires=");
+		smart_str_append(&buf, dt);
+		smart_str_appends(&buf, "; Max-Age=0");
 		zend_string_free(dt);
 	} else {
-		snprintf(cookie, len + 100, "Set-Cookie: %s=%s", ZSTR_VAL(name), value ? ZSTR_VAL(encoded_value) : "");
+		smart_str_appends(&buf, "Set-Cookie: ");
+		smart_str_append(&buf, name);
+		smart_str_appendc(&buf, '=');
+		if (url_encode) {
+			zend_string *encoded_value = php_raw_url_encode(ZSTR_VAL(value), ZSTR_LEN(value));
+			smart_str_append(&buf, encoded_value);
+			zend_string_release_ex(encoded_value, 0);
+		} else {
+			smart_str_append(&buf, value);
+		}
 		if (expires > 0) {
 			const char *p;
-			char tsdelta[13];
 			double diff;
 
-			strlcat(cookie, COOKIE_EXPIRES, len + 100);
+			smart_str_appends(&buf, COOKIE_EXPIRES);
 			dt = php_format_date("D, d-M-Y H:i:s T", sizeof("D, d-M-Y H:i:s T")-1, expires, 0);
 			/* check to make sure that the year does not exceed 4 digits in length */
 			p = zend_memrchr(ZSTR_VAL(dt), '-', ZSTR_LEN(dt));
 			if (!p || *(p + 5) != ' ') {
 				zend_string_free(dt);
-				efree(cookie);
-				zend_string_release(encoded_value);
+				smart_str_free(&buf);
 				zend_error(E_WARNING, "Expiry date cannot have a year greater than 9999");
 				return FAILURE;
 			}
-			strlcat(cookie, ZSTR_VAL(dt), len + 100);
+
+			smart_str_append(&buf, dt);
 			zend_string_free(dt);
 
 			diff = difftime(expires, php_time());
 			if (diff < 0) {
 				diff = 0;
 			}
-			snprintf(tsdelta, sizeof(tsdelta), ZEND_LONG_FMT, (zend_long) diff);
-			strlcat(cookie, COOKIE_MAX_AGE, len + 100);
-			strlcat(cookie, tsdelta, len + 100);
+
+			smart_str_appends(&buf, COOKIE_MAX_AGE);
+			smart_str_append_long(&buf, (zend_long) diff);
 		}
 	}
 
-	if (encoded_value) {
-		zend_string_release(encoded_value);
-	}
-
 	if (path && ZSTR_LEN(path)) {
-		strlcat(cookie, COOKIE_PATH, len + 100);
-		strlcat(cookie, ZSTR_VAL(path), len + 100);
+		smart_str_appends(&buf, COOKIE_PATH);
+		smart_str_append(&buf, path);
 	}
 	if (domain && ZSTR_LEN(domain)) {
-		strlcat(cookie, COOKIE_DOMAIN, len + 100);
-		strlcat(cookie, ZSTR_VAL(domain), len + 100);
+		smart_str_appends(&buf, COOKIE_DOMAIN);
+		smart_str_append(&buf, domain);
 	}
 	if (secure) {
-		strlcat(cookie, COOKIE_SECURE, len + 100);
+		smart_str_appends(&buf, COOKIE_SECURE);
 	}
 	if (httponly) {
-		strlcat(cookie, COOKIE_HTTPONLY, len + 100);
+		smart_str_appends(&buf, COOKIE_HTTPONLY);
+	}
+	if (samesite && ZSTR_LEN(samesite)) {
+		smart_str_appends(&buf, COOKIE_SAMESITE);
+		smart_str_append(&buf, samesite);
 	}
 
-	ctr.line = cookie;
-	ctr.line_len = (uint32_t)strlen(cookie);
+	ctr.line = ZSTR_VAL(buf.s);
+	ctr.line_len = (uint32_t) ZSTR_LEN(buf.s);
 
 	result = sapi_header_op(SAPI_HEADER_ADD, &ctr);
-	efree(cookie);
+	zend_string_release(buf.s);
 	return result;
 }
 
+static void php_head_parse_cookie_options_array(zval *options, zend_long *expires, zend_string **path, zend_string **domain, zend_bool *secure, zend_bool *httponly, zend_string **samesite) {
+	int found = 0;
+	zend_string *key;
+	zval *value;
+
+	ZEND_HASH_FOREACH_STR_KEY_VAL(Z_ARRVAL_P(options), key, value) {
+		if (key) {
+			if (zend_string_equals_literal_ci(key, "expires")) {
+				*expires = zval_get_long(value);
+				found++;
+			} else if (zend_string_equals_literal_ci(key, "path")) {
+				*path = zval_get_string(value);
+				found++;
+			} else if (zend_string_equals_literal_ci(key, "domain")) {
+				*domain = zval_get_string(value);
+				found++;
+			} else if (zend_string_equals_literal_ci(key, "secure")) {
+				*secure = zval_is_true(value);
+				found++;
+			} else if (zend_string_equals_literal_ci(key, "httponly")) {
+				*httponly = zval_is_true(value);
+				found++;
+			} else if (zend_string_equals_literal_ci(key, "samesite")) {
+				*samesite = zval_get_string(value);
+				found++;
+			} else {
+				php_error_docref(NULL, E_WARNING, "Unrecognized key '%s' found in the options array", ZSTR_VAL(key));
+			}
+		} else {
+			php_error_docref(NULL, E_WARNING, "Numeric key found in the options array");
+		}
+	} ZEND_HASH_FOREACH_END();
+
+	/* Array is not empty but no valid keys were found */
+	if (found == 0 && zend_hash_num_elements(Z_ARRVAL_P(options)) > 0) {
+		php_error_docref(NULL, E_WARNING, "No valid options were found in the given array");
+	}
+}
 
-/* php_set_cookie(name, value, expires, path, domain, secure) */
 /* {{{ proto bool setcookie(string name [, string value [, int expires [, string path [, string domain [, bool secure[, bool httponly]]]]]])
+                  setcookie(string name [, string value [, array options]])
    Send a cookie */
 PHP_FUNCTION(setcookie)
 {
-	zend_string *name, *value = NULL, *path = NULL, *domain = NULL;
+	zval *expires_or_options = NULL;
+	zend_string *name, *value = NULL, *path = NULL, *domain = NULL, *samesite = NULL;
 	zend_long expires = 0;
 	zend_bool secure = 0, httponly = 0;
 
@@ -205,26 +239,54 @@ PHP_FUNCTION(setcookie)
 		Z_PARAM_STR(name)
 		Z_PARAM_OPTIONAL
 		Z_PARAM_STR(value)
-		Z_PARAM_LONG(expires)
+		Z_PARAM_ZVAL(expires_or_options)
 		Z_PARAM_STR(path)
 		Z_PARAM_STR(domain)
 		Z_PARAM_BOOL(secure)
 		Z_PARAM_BOOL(httponly)
 	ZEND_PARSE_PARAMETERS_END();
 
-	if (php_setcookie(name, value, expires, path, domain, secure, 1, httponly) == SUCCESS) {
-		RETVAL_TRUE;
-	} else {
-		RETVAL_FALSE;
+	if (expires_or_options) {
+		if (Z_TYPE_P(expires_or_options) == IS_ARRAY) {
+			if (UNEXPECTED(ZEND_NUM_ARGS() > 3)) {
+				php_error_docref(NULL, E_WARNING, "Cannot pass arguments after the options array");
+				RETURN_FALSE;
+			}
+			php_head_parse_cookie_options_array(expires_or_options, &expires, &path, &domain, &secure, &httponly, &samesite);
+		} else {
+			expires = zval_get_long(expires_or_options);
+		}
+	}
+
+	if (!EG(exception)) {
+		if (php_setcookie(name, value, expires, path, domain, secure, httponly, samesite, 1) == SUCCESS) {
+			RETVAL_TRUE;
+		} else {
+			RETVAL_FALSE;
+		}
+	}
+
+	if (expires_or_options && Z_TYPE_P(expires_or_options) == IS_ARRAY) {
+		if (path) {
+			zend_string_release(path);
+		}
+		if (domain) {
+			zend_string_release(domain);
+		}
+		if (samesite) {
+			zend_string_release(samesite);
+		}
 	}
 }
 /* }}} */
 
 /* {{{ proto bool setrawcookie(string name [, string value [, int expires [, string path [, string domain [, bool secure[, bool httponly]]]]]])
+                  setrawcookie(string name [, string value [, array options]])
    Send a cookie with no url encoding of the value */
 PHP_FUNCTION(setrawcookie)
 {
-	zend_string *name, *value = NULL, *path = NULL, *domain = NULL;
+	zval *expires_or_options = NULL;
+	zend_string *name, *value = NULL, *path = NULL, *domain = NULL, *samesite = NULL;
 	zend_long expires = 0;
 	zend_bool secure = 0, httponly = 0;
 
@@ -232,17 +294,43 @@ PHP_FUNCTION(setrawcookie)
 		Z_PARAM_STR(name)
 		Z_PARAM_OPTIONAL
 		Z_PARAM_STR(value)
-		Z_PARAM_LONG(expires)
+		Z_PARAM_ZVAL(expires_or_options)
 		Z_PARAM_STR(path)
 		Z_PARAM_STR(domain)
 		Z_PARAM_BOOL(secure)
 		Z_PARAM_BOOL(httponly)
 	ZEND_PARSE_PARAMETERS_END();
 
-	if (php_setcookie(name, value, expires, path, domain, secure, 0, httponly) == SUCCESS) {
-		RETVAL_TRUE;
-	} else {
-		RETVAL_FALSE;
+	if (expires_or_options) {
+		if (Z_TYPE_P(expires_or_options) == IS_ARRAY) {
+			if (UNEXPECTED(ZEND_NUM_ARGS() > 3)) {
+				php_error_docref(NULL, E_WARNING, "Cannot pass arguments after the options array");
+				RETURN_FALSE;
+			}
+			php_head_parse_cookie_options_array(expires_or_options, &expires, &path, &domain, &secure, &httponly, &samesite);
+		} else {
+			expires = zval_get_long(expires_or_options);
+		}
+	}
+
+	if (!EG(exception)) {
+		if (php_setcookie(name, value, expires, path, domain, secure, httponly, samesite, 0) == SUCCESS) {
+			RETVAL_TRUE;
+		} else {
+			RETVAL_FALSE;
+		}
+	}
+
+	if (expires_or_options && Z_TYPE_P(expires_or_options) == IS_ARRAY) {
+		if (path) {
+			zend_string_release(path);
+		}
+		if (domain) {
+			zend_string_release(domain);
+		}
+		if (samesite) {
+			zend_string_release(samesite);
+		}
 	}
 }
 /* }}} */
@@ -258,8 +346,8 @@ PHP_FUNCTION(headers_sent)
 
 	ZEND_PARSE_PARAMETERS_START(0, 2)
 		Z_PARAM_OPTIONAL
-		Z_PARAM_ZVAL_DEREF(arg1)
-		Z_PARAM_ZVAL_DEREF(arg2)
+		Z_PARAM_ZVAL(arg1)
+		Z_PARAM_ZVAL(arg2)
 	ZEND_PARSE_PARAMETERS_END();
 
 	if (SG(headers_sent)) {
@@ -269,14 +357,12 @@ PHP_FUNCTION(headers_sent)
 
 	switch(ZEND_NUM_ARGS()) {
 	case 2:
-		zval_ptr_dtor(arg2);
-		ZVAL_LONG(arg2, line);
+		ZEND_TRY_ASSIGN_REF_LONG(arg2, line);
 	case 1:
-		zval_ptr_dtor(arg1);
 		if (file) {
-			ZVAL_STRING(arg1, file);
+			ZEND_TRY_ASSIGN_REF_STRING(arg1, file);
 		} else {
-			ZVAL_EMPTY_STRING(arg1);
+			ZEND_TRY_ASSIGN_REF_EMPTY_STRING(arg1);
 		}
 		break;
 	}
@@ -313,7 +399,7 @@ PHP_FUNCTION(headers_list)
 }
 /* }}} */
 
-/* {{{ proto long http_response_code([int response_code])
+/* {{{ proto int http_response_code([int response_code])
    Sets a response code, or returns the current HTTP response code */
 PHP_FUNCTION(http_response_code)
 {
@@ -345,11 +431,3 @@ PHP_FUNCTION(http_response_code)
 	RETURN_LONG(SG(sapi_headers).http_response_code);
 }
 /* }}} */
-
-/*
- * Local variables:
- * tab-width: 4
- * c-basic-offset: 4
- * vim600: sw=4 ts=4 fdm=marker
- * vim<600: sw=4 ts=4 * End:
- */
