@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 7                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2018 The PHP Group                                |
+   | Copyright (c) The PHP Group                                          |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -16,8 +16,6 @@
    +----------------------------------------------------------------------+
 */
 
-/* $Id$ */
-
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -59,7 +57,7 @@ ZEND_END_ARG_INFO()
  *
  * Every user visible function must have an entry in tokenizer_functions[].
  */
-const zend_function_entry tokenizer_functions[] = {
+static const zend_function_entry tokenizer_functions[] = {
 	PHP_FE(token_get_all,	arginfo_token_get_all)
 	PHP_FE(token_name,		arginfo_token_name)
 	PHP_FE_END
@@ -112,7 +110,11 @@ static void add_token(zval *return_value, int token_type,
 		zval keyword;
 		array_init(&keyword);
 		add_next_index_long(&keyword, token_type);
-		add_next_index_stringl(&keyword, (char *) text, leng);
+		if (leng == 1) {
+			add_next_index_str(&keyword, ZSTR_CHAR(text[0]));
+		} else {
+			add_next_index_stringl(&keyword, (char *) text, leng);
+		}
 		add_next_index_long(&keyword, lineno);
 		add_next_index_zval(return_value, &keyword);
 	} else {
@@ -144,12 +146,11 @@ static zend_bool tokenize(zval *return_value, zend_string *source)
 	LANG_SCNG(yy_state) = yycINITIAL;
 	array_init(return_value);
 
-	ZVAL_UNDEF(&token);
-	while ((token_type = lex_scan(&token))) {
+	while ((token_type = lex_scan(&token, NULL))) {
 		add_token(return_value, token_type, zendtext, zendleng, token_line);
 
 		if (Z_TYPE(token) != IS_UNDEF) {
-			zval_dtor(&token);
+			zval_ptr_dtor_nogc(&token);
 			ZVAL_UNDEF(&token);
 		}
 
@@ -178,7 +179,7 @@ static zend_bool tokenize(zval *return_value, zend_string *source)
 		token_line = CG(zend_lineno);
 	}
 
-	zval_dtor(&source_zval);
+	zval_ptr_dtor_str(&source_zval);
 	zend_restore_lexical_state(&original_lex_state);
 
 	return 1;
@@ -192,8 +193,16 @@ void on_event(zend_php_scanner_event event, int token, int line, void *context)
 
 	switch (event) {
 		case ON_TOKEN:
-			if (token == END) break;
-			add_token(token_stream, token, LANG_SCNG(yy_text), LANG_SCNG(yy_leng), line);
+			{
+				if (token == END) break;
+				/* Special cases */
+				if (token == ';' && LANG_SCNG(yy_leng) > 1) { /* ?> or ?>\n or ?>\r\n */
+					token = T_CLOSE_TAG;
+				} else if (token == T_ECHO && LANG_SCNG(yy_leng) == sizeof("<?=") - 1) {
+					token = T_OPEN_TAG_WITH_ECHO;
+				}
+				add_token(token_stream, token, LANG_SCNG(yy_text), LANG_SCNG(yy_leng), line);
+			}
 			break;
 		case ON_FEEDBACK:
 			tokens_ht = Z_ARRVAL_P(token_stream);
@@ -248,7 +257,7 @@ static zend_bool tokenize_parse(zval *return_value, zend_string *source)
 	zend_restore_lexical_state(&original_lex_state);
 	CG(in_compilation) = original_in_compilation;
 
-	zval_dtor(&source_zval);
+	zval_ptr_dtor_str(&source_zval);
 
 	return success;
 }
@@ -294,12 +303,3 @@ PHP_FUNCTION(token_name)
 	RETVAL_STRING(get_token_type_name(type));
 }
 /* }}} */
-
-/*
- * Local variables:
- * tab-width: 4
- * c-basic-offset: 4
- * End:
- * vim600: noet sw=4 ts=4 fdm=marker
- * vim<600: noet sw=4 ts=4
- */
