@@ -114,8 +114,7 @@ static int fpm_event_port_clean() /* {{{ */
  */
 static int fpm_event_port_wait(struct fpm_event_queue_s *queue, unsigned long int timeout) /* {{{ */
 {
-	int ret;
-	unsigned int i, nget;
+	int ret, i, nget;
 	timespec_t t;
 
 	/* convert timeout into timespec_t */
@@ -124,8 +123,6 @@ static int fpm_event_port_wait(struct fpm_event_queue_s *queue, unsigned long in
 
 	/* wait for inconming event or timeout. We want at least one event or timeout */
 	nget = 1;
-	events[0].portev_user = (void *)-1; /* so we can double check that an event was returned */
-
 	ret = port_getn(pfd, events, nevents, &nget, &t);
 	if (ret < 0) {
 
@@ -133,16 +130,6 @@ static int fpm_event_port_wait(struct fpm_event_queue_s *queue, unsigned long in
 		if (errno != EINTR && errno != ETIME) {
 			zlog(ZLOG_WARNING, "poll() returns %d", errno);
 			return -1;
-		} else if (nget > 0 && events[0].portev_user == (void *)-1) {
-			/* This confusing API can return an event at the same time
-			 * that it reports EINTR or ETIME.  If that occurs, just
-			 * report the event.  With EINTR, nget can be > 0 without
-			 * any event, so check that portev_user was filled in.
-			 *
-			 * See discussion thread
-			 *   http://marc.info/?l=opensolaris-networking-discuss&m=125071205204540
-			 */
-			nget = 0;
 		}
 	}
 
